﻿using System;
using System.Linq;
using System.Collections.Generic;
using HeuristicLab.Persistence.Auxiliary;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Persistence.Interfaces;

namespace HeuristicLab.Persistence.Core {

  [StorableClass]
  internal sealed class CachedTypeSerializer : ICompositeSerializer {

    [StorableConstructor]
    private CachedTypeSerializer(bool deserializing) { }

    private TypeCache typeCache;
    private Dictionary<int, Type> deserializationTypeCache;

    internal CachedTypeSerializer(TypeCache typeCache) {
      this.typeCache = typeCache;
    }

    internal CachedTypeSerializer(Dictionary<int, Type> deserializationTypeCache) {
      this.deserializationTypeCache = deserializationTypeCache;
    }

    public int Priority {
      get { return 1000; }
    }

    public bool CanSerialize(Type type) {
      return type == typeof(Type) || type.VersionInvariantName() == "System.RuntimeType, mscorlib";
    }

    public string JustifyRejection(Type type) {
      return "not System.Type nor System.RuntimeType";
    }

    public IEnumerable<Tag> CreateMetaInfo(object o) {
      yield return new Tag("TypeId", typeCache.GetOrCreateId((Type)o));
    }

    public IEnumerable<Tag> Decompose(object obj) {
      return new Tag[] { };
    }

    public object CreateInstance(Type type, IEnumerable<Tag> metaInfo) {
      int typeId = -1;
      try {
        typeId = (int)metaInfo.First().Value;
        return deserializationTypeCache[typeId];
      } catch (InvalidOperationException e) {
        throw new PersistenceException("Insufficient meta information to instantiate Type object", e);
      } catch (InvalidCastException e) {
        throw new PersistenceException("Invalid meta information during reconstruction of Type object", e);
      } catch (KeyNotFoundException e) {
        throw new PersistenceException(String.Format(
          "TypeCache has not been loaded with type id {0}, make sure all required assemblies are available.",
          typeId), e);
      }
    }

    public void Populate(object instance, IEnumerable<Tag> objects, Type type) {
      // Type ojects are populated during instance creation.
    }
  }
}
